/* Nessus Attack Scripting Language 
 *
 * Copyright (C) 2002 - 2003 Michel Arboi and Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * In addition, as a special exception, Renaud Deraison and Michel Arboi
 * give permission to link the code of this program with any
 * version of the OpenSSL library which is distributed under a
 * license identical to that listed in the included COPYING.OpenSSL
 * file, and distribute linked combinations including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * this file, you may extend this exception to your version of the
 * file, but you are not obligated to do so.  If you do not wish to
 * do so, delete this exception statement from your version.
 *
 */
#include <includes.h>


#include "nasl_tree.h"
#include "nasl_global_ctxt.h"
#include "nasl_func.h"
#include "nasl_var.h"
#include "nasl_lex_ctxt.h"
#include "exec.h"

#ifndef NASL_DEBUG
#define NASL_DEBUG 0
#endif

static int
hash_int(int x)
{
  return (unsigned) x % VAR_NAME_HASH;
}

static int
hash_str(const char* s)
{
  const char	*p;
  int		h = 0;

  if (s == NULL)
    return 0;
  for (p = s; *p != '\0'; p ++)
    h = (h * (VAR_NAME_HASH-1) + *p) % VAR_NAME_HASH;
  return h;
}


nasl_var*
nasl_get_var_by_num(nasl_var** vars, int num, int create)
{
  int		h = hash_int(num);
  nasl_var	*v;

  for (v = vars[h]; v != NULL; v = v->next_var)
    if (v->var_name == NULL && num == v->var_num)
      return v;

  if (! create)
    return NULL;

  v = emalloc(sizeof(nasl_var));
  v->var_num = num;
  v->var_type = VAR2_UNDEF;
  v->next_var = vars[h];

  vars[h] = v;
  return v;  
}

static nasl_var*
get_var_by_name(nasl_var** vars, const char* s)
{
  int		h = hash_str(s);
  nasl_var	*v;
  
  for (v = vars[h]; v != NULL; v = v->next_var)
    if (v->var_name != NULL && strcmp(s, v->var_name) == 0)
      return v;

  v = emalloc(sizeof(nasl_var));
  v->var_name = estrdup(s);
  v->var_type = VAR2_UNDEF;
  v->next_var = vars[h];

  vars[h] = v;
  return v;  
}



/* This function climbs up in the context list */
static nasl_var*
get_var_ref_by_name(lex_ctxt* ctxt, const char* name, int climb)
{
  nasl_var	*v;
  char		*p;
  int	h = hash_str(name);
  lex_ctxt	*c;
  harglst	*globals, * variables, *types;
  int		vtype;

  if(climb != 0)
  {
  for (c = ctxt; c != NULL; c = c->up_ctxt)
    {
      for (v = c->variables[h]; v != NULL; v = v->next_var)
	if (v->var_name != NULL && strcmp(name, v->var_name) == 0)
	  return v;
    }
  }
 else
 {
   for (v = ctxt->variables[h]; v != NULL; v = v->next_var)
	if (v->var_name != NULL && strcmp(name, v->var_name) == 0)
	  return v;
 }

  v = emalloc(sizeof(nasl_var));
  v->var_name = estrdup(name);
  v->var_type = VAR2_UNDEF;
  v->next_var = ctxt->variables[h];
  ctxt->variables[h] = v;

  return v;  
}


static nasl_var*
get_var_ref_by_num(lex_ctxt* ctxt, int num)
{
  nasl_var	*v;
  int	h = hash_int(num);

  /* We do not climb the contexts, this does not make sense */
  for (v = ctxt->variables[h]; v != NULL; v = v->next_var)
    if (v->var_name == NULL && v->var_num == num)
      return v;

  v = emalloc(sizeof(nasl_var));
  v->var_num = num;
  v->var_type = VAR2_UNDEF;
  v->next_var = ctxt->variables[h];

  ctxt->variables[h] = v;
  return v;  
}

static tree_cell*
var2cell(nasl_var* v)
{
  tree_cell	*tc = alloc_tree_cell(0, NULL);

  tc->type = REF_VAR;
  tc->x.ref_val = v;		/* No need to free this later! */
  return tc;
}

tree_cell*
get_variable_by_name(lex_ctxt* ctxt, const char* name)
{
  nasl_var	* v;

  if (name == NULL)
    return NULL;
  v = get_var_ref_by_name(ctxt, name, 1);
  return var2cell(v);
}

const char*
get_var_name(nasl_var *v)
{
  static char	n[32];

  if (v->var_name != NULL)
    return v->var_name;
  sprintf(n, "#%d", v->var_num);
  return n;
}

tree_cell*
get_array_elem(lex_ctxt* ctxt, const char* name, tree_cell* idx)
{
  nasl_var	*a = get_var_ref_by_name(ctxt, name, 1);
  nasl_var	*e;
  char		txt[32], *idxname;
  int		h;
  tree_cell	*tc;
  lex_ctxt	lexic2;


  if (idx == NULL)
    {
#if NASL_DEBUG > 0
      fprintf(stderr, "get_array_elem: NULL index\n");
#endif
      return NULL;
    }

  switch (a->var_type)
    {
    case VAR2_UNDEF:
      /* We define the array here */
      a->var_type = VAR2_ARRAY;
    case VAR2_ARRAY:
      if (a->v.a_val == NULL)
	a->v.a_val = emalloc(VAR_NAME_HASH * sizeof(nasl_var*));

      switch(idx->type)
	{
	case CONST_INT:
	  e = nasl_get_var_by_num(a->v.a_val, idx->x.i_val, 1);
	  return var2cell(e);
	  
	case CONST_STR:
	case CONST_DATA:
	  e = get_var_by_name(a->v.a_val, idx->x.str_val);
	  return var2cell(e);
	  
	default:
	  fprintf(stderr, "get_array_elem: unhandled index type 0x%x\n",
		  idx->type);
	  return NULL;
	}
      /*NOTREACHED*/
      break;

    case VAR2_INT:
      fprintf(stderr, "get_array_elem: variable %s is an integer\n",
	      get_var_name(a));
      return NULL;

    case VAR2_STRING:
    case VAR2_DATA:
      if (idx->type == CONST_INT)
	{
	  int	l = a->var_size;

	  if (idx->x.i_val >= l)
	    {
	      fprintf(stderr, "get_array_elem: requesting character after end of string %s (%d >= %d)\n", get_var_name(a), idx->x.i_val, l);
	      tc = alloc_expr_cell(idx->line_nb, CONST_DATA /*CONST_STR*/,
				   NULL, NULL);
	      tc->x.str_val = estrdup("");
	      tc->size = 0;
	      return tc;
	    }
	  else
	    {
	      tc = alloc_expr_cell(idx->line_nb, CONST_DATA /*CONST_STR*/,
				   NULL, NULL);
	      tc->x.str_val = emalloc(2);
	      tc->x.str_val[0] = a->v.str_val[idx->x.i_val];
	      tc->x.str_val[1] = '\0';
	      tc->size = 1;
	      return tc;
	    }
	}
      else
	{
	  fprintf(stderr, "get_array_elem: Cannot use a non integer index  (type 0x%x) in string\n", idx->type);
	  return NULL;
	}
      /*NOTREACHED*/
      break;

    default:
      fprintf(stderr, "Severe bug: unknown variable type 0x%x %s\n",
	      a->var_type, get_line_nb(idx));
      return NULL;
    }
  /*NOTREACHED*/
  return NULL;
}


void
free_var_chain(nasl_var* v)
{
  int	i;

  if (v == NULL)
    return;
  free_var_chain(v->next_var);
  efree(&v->var_name);
  switch(v->var_type)
    {
    case VAR2_STRING:
    case VAR2_DATA:
      efree(&v->v.str_val);
      break;
    case VAR2_ARRAY:
      for (i = 0; i < VAR_NAME_HASH; i ++)
	free_var_chain(v->v.a_val[i]);
      efree(&v->v.a_val);
      break;
    }
  efree(&v);
}

void
clear_var(nasl_var* v)
{
  int	i;

  if (v == NULL)
    return;

  switch (v->var_type)
    {
    case VAR2_INT:
      v->v.i_val = 0;
      break;
    case VAR2_STRING:
    case VAR2_DATA:
      efree(&v->v.str_val);
      break;
    case VAR2_ARRAY:
      for (i = 0; i < VAR_NAME_HASH; i ++)
	free_var_chain(v->v.a_val[i]);
      efree(&v->v.a_val);
      break;
    }
  v->var_type = VAR2_UNDEF;
  return;
}


static nasl_var*
dup_var(const nasl_var* v)
{
  nasl_var	*v2;

  if (v == NULL)
    return NULL;

  v2 = emalloc(sizeof(nasl_var));
  v2->var_type = v->var_type;
  v2->var_num = v->var_num;
  if (v->var_name != NULL)
    v2->var_name = estrdup(v->var_name);

  switch(v->var_type)
    {
    case VAR2_STRING:
    case VAR2_DATA:
      v2->v.str_val = emalloc(v->var_size);
      memcpy(v2->v.str_val, v->v.str_val, v->var_size+1);
      v2->var_size = v->var_size;
      break;

    case VAR2_UNDEF:
      break;

    case VAR2_INT:
      v2->v.i_val = v->v.i_val;
      break;

    case VAR2_ARRAY:
    default:
      fprintf(stderr, "dup_var: unhandled type 0x%x\n", v->var_type);
      efree(&v);
      return NULL;
    }
  return v2;
}

static nasl_var*
dup_var_chain(const nasl_var* v)
{
  int	i;
  nasl_var	*v2;

  v2 = dup_var(v);
  if (v2 != NULL)
    v2->next_var = dup_var_chain(v->next_var);
  return v2;
}


extern FILE*	nasl_trace_fp;

tree_cell*
nasl_affect(tree_cell* lval, tree_cell* rval)
{
  nasl_var	*v1 = NULL, *v2 = NULL, **pv = NULL;
  int		t1, t2, i;


  if (lval->type != REF_VAR)
    {
      fprintf(stderr, "nasl_affect: cannot affect to non variable %s\n",
	      nasl_type_name(lval->type));
      return NULL;
    }

  v1 = lval->x.ref_val;
  t1 = v1->var_type;

  if (rval == NULL || rval == FAKE_CELL)
    {
#if NASL_DEBUG > 0
      fprintf(stderr, "nasl_affect: cannot affect NULL or FAKE cell - undefining variable '%s' %s\n", get_var_name(v1), get_line_nb(lval));
#endif
      clear_var(v1);
      if (nasl_trace_fp != NULL)
	fprintf(nasl_trace_fp, "NASL> %s <- undef\n", get_var_name(v1));

      return NULL;
    }

    switch (rval->type)
      {
      case CONST_INT:
	t2 = VAR2_INT;
	break;
      case CONST_STR:
	t2 = VAR2_STRING;
	break;
      case CONST_DATA:
	t2 = VAR2_DATA;
	break;

      case REF_VAR:
	v2 = rval->x.ref_val;
	pv = v2->v.a_val;
	t2 = v2->var_type;
	break;

      case REF_ARRAY:
      case DYN_ARRAY:
	pv = rval->x.ref_val;
	t2 = VAR2_ARRAY;
	break;

      default:
	fprintf(stderr, "Cannot affect rvalue 0x%x to variable\n", rval->type);
	return NULL;
      }

  clear_var(v1);

  if (t1 != VAR2_UNDEF && t2 == VAR2_UNDEF)
    {
#if NASL_DEBUG > 0
      fprintf(stderr, 
	      "Warning: Undefining defined variable %s\n",get_var_name(v1));
#endif
    }
  else if (t1 == VAR2_ARRAY && t2 != VAR2_ARRAY)
    {
#if NASL_DEBUG > 1
      fprintf(stderr, 
	      "Warning: affecting non array (0x%x) to array variable %s\n",
	      t2, get_var_name(v1));
#endif
    }
  else if ((t1 == VAR2_INT || t1 == VAR2_STRING || t1 == VAR2_DATA) &&
	   t2 == VAR2_ARRAY)
    {
#if NASL_DEBUG > 1
      fprintf(stderr, "Warning: affecting array to atomic variable (0x%x)\n",
	      get_var_name(v1), t2);
#endif
    }

  v1->var_type = t2;
  if (rval->type != REF_VAR && rval->type != REF_ARRAY && rval->type != DYN_ARRAY)
    switch (t2)
      {
      case VAR2_INT:
	v1->v.i_val = rval->x.i_val;
	break;
      case VAR2_STRING:
      case VAR2_DATA:
	v1->v.str_val = emalloc(rval->size);
	bcopy(rval->x.str_val, v1->v.str_val, rval->size);
	v1->var_size = rval->size;
	break;
      }
  else				/* REF_xxx */
    switch(t2)
      {
      case VAR2_INT:
	v1->v.i_val = v2->v.i_val;
	break;
      case VAR2_STRING:
      case VAR2_DATA:
	v1->v.str_val = emalloc(v2->var_size);
	bcopy(v2->v.str_val, v1->v.str_val, v2->var_size);
	v1->var_size = v2->var_size;
	break;
      case VAR2_ARRAY:
	v1->v.a_val = emalloc(VAR_NAME_HASH * sizeof(nasl_var*));
	for (i = 0; i < VAR_NAME_HASH; i ++)
	  v1->v.a_val[i] = dup_var_chain(pv[i]);
	break;
      }

  if (nasl_trace_fp != NULL)
    switch(t2)
      {
      case VAR2_INT:
	fprintf(nasl_trace_fp, "NASL> %s <- %d\n", get_var_name(v1), v1->v.i_val);
	break;
      case VAR2_STRING:
      case VAR2_DATA:
	fprintf(nasl_trace_fp, "NASL> %s <- \"%s\"\n", get_var_name(v1), v1->v.str_val);
	break;
      case VAR2_ARRAY:
	fprintf(nasl_trace_fp, "NASL> %s <- (VAR2_ARRAY)\n", get_var_name(v1));
	break;
      default:
	fprintf(nasl_trace_fp, "NASL> %s <- (Type 0x%x)\n",
		get_var_name(v1), t2);
	break;
      }

  return FAKE_CELL;
}


static nasl_var*
create_var(const char* name, tree_cell* val)
{
  nasl_var	*v = emalloc(sizeof(nasl_var)), *v2, **pv;
  int		t2, i;

  if (name != NULL)
    v->var_name = estrdup(name);

  if (val == NULL || val == FAKE_CELL)
    {
#if NASL_DEBUG > 0
      fprintf(stderr, "create_var: should not affect NULL or FAKE cell to variable\n");
#endif
      v->var_type = VAR2_UNDEF;
      return v;
    }

  /* Duplicated code - should be cleaned! */
  switch (val->type)
    {
    case CONST_INT:
      v->var_type = VAR2_INT;
      v->v.i_val = val->x.i_val;
      break;
    case CONST_STR:
      v->var_type = VAR2_STRING;
      /* Fix bad string length */
      if (val->size <= 0)
	v->var_size = strlen(val->x.str_val);
      else
	v->var_size = val->size;
      v->v.str_val = emalloc(val->size + 1);
      memcpy(v->v.str_val, val->x.str_val, val->size);
      break;
    case CONST_DATA:
      v->var_type = VAR2_DATA;
      v->var_size = val->size;
      v->v.str_val = emalloc(val->size);
      memcpy(v->v.str_val, val->x.str_val, val->size);
      break;
    case REF_ARRAY:
    case DYN_ARRAY:
      v->var_type = VAR2_ARRAY;
      pv = val->x.ref_val;
      if (pv != NULL)
	{
	  v->v.a_val = emalloc(sizeof(nasl_var*) * VAR_NAME_HASH);
	  for (i = 0; i < VAR_NAME_HASH; i ++)
	    v->v.a_val[i] = dup_var_chain(pv[i]);
	}
      break;

    case REF_VAR:
      v2 = val->x.ref_val;
      fprintf(stderr, "create_var: copying REF_VAR to %s\n", get_var_name(v2));
      v->var_type = v2->var_type;
      v->v = v2->v;
      break;

    default:
      fprintf(stderr, "create_var: cannot affect rvalue 0x%x to variable\n",
	      val->type);
#if 1
      v->var_type = VAR2_UNDEF;
#else
      efree(&v);
      return NULL;
#endif
    }
  return v;
}

tree_cell* decl_local_variables(lex_ctxt* lexic, tree_cell* vars)
{
  tree_cell	*t;

  for (t = vars; t != NULL; t = t->link[0])
    if (t->x.str_val == NULL)
      fprintf(stderr, "decl_local_variables: null name!\n");
    else
      add_named_var_to_ctxt(lexic, t->x.str_val, NULL);
  return FAKE_CELL;
}

tree_cell*
decl_global_variables(lex_ctxt* lexic, tree_cell* vars)
{
  lex_ctxt	*c = lexic;

  while (c->up_ctxt != NULL)
    c = c->up_ctxt;
  return decl_local_variables(c, vars);
}

nasl_var*
add_numbered_var_to_ctxt(lex_ctxt* lexic, int num, tree_cell* val)
{
  int		h = hash_int(num);
  nasl_var	*v;
  
  for (v = lexic->variables[h]; v != NULL; v = v->next_var)
    if (v->var_name == NULL && num == v->var_num)
      {
	fprintf(stderr, "Cannot add existing variable %d\n", num);
	return NULL;
      }
  v = create_var(NULL, val);
  v->var_num = num;
  v->next_var = lexic->variables[h];
  lexic->variables[h] = v;
  return v;
}

nasl_var*
add_named_var_to_ctxt(lex_ctxt* lexic, const char* name, tree_cell* val)
{
  int		h = hash_str(name);
  nasl_var	*v;
  
  /* Duplicated code - again */
  for (v = lexic->variables[h]; v != NULL; v = v->next_var)
    if (v->var_name != NULL && strcmp(name, v->var_name) == 0)
      {
	fprintf(stderr, "Cannot add existing variable %s\n", name);
	return NULL;
      }
  v = create_var(name, val);
  if (v == NULL)
    return NULL;
  v->next_var = lexic->variables[h];
  lexic->variables[h] = v;
  return v;
}



tree_cell*
nasl_read_var_ref(lex_ctxt* lexic, tree_cell* tc)
{
  tree_cell	*ret;
  nasl_var	*v;

  if (tc == NULL || tc == FAKE_CELL)
    {
      fprintf(stderr, "nasl_read_var_ref: cannot read NULL or FAKE cell\n");
      return NULL;
    }
  if (tc->type != REF_VAR)
    {
      fprintf(stderr,
	      "nasl_read_var_ref: argument (type=%d) is not REF_VAR %s\n",
	      tc->type, get_line_nb(tc));
      return NULL;
    }

  v = tc->x.ref_val;
  ret = alloc_tree_cell(tc->line_nb, NULL);

  if (nasl_trace_fp != NULL)
    fprintf(nasl_trace_fp, "NASL> %s -> ", get_var_name(v));

  switch (v->var_type)
    {
    case VAR2_INT:
      ret->type = CONST_INT;
      ret->x.i_val = v->v.i_val;
      if (nasl_trace_fp != NULL)
	fprintf(nasl_trace_fp, "%d\n", ret->x.i_val);
      return ret;

    case VAR2_STRING:
      ret->type = CONST_STR;
      ret->x.str_val = estrdup(v->v.str_val);
      /* Fix bad string length */
      if (v->var_size <= 0)
	ret->size = strlen(v->v.str_val);
      else
	ret->size = v->var_size;
      if (nasl_trace_fp != NULL)
	fprintf(nasl_trace_fp, "\"%s\"\n", ret->x.str_val);
      return ret;

    case VAR2_DATA:
      ret->type = CONST_DATA;
      ret->x.str_val = emalloc(v->var_size);
      memcpy(ret->x.str_val, v->v.str_val, v->var_size);
      ret->size = v->var_size;
      if (nasl_trace_fp != NULL)
	fprintf(nasl_trace_fp, "\"%s\"\n", ret->x.str_val);
      return ret;

    case VAR2_ARRAY:
      ret->type = REF_ARRAY;
      ret->x.ref_val = v->v.a_val;
      return ret;

    case VAR2_UNDEF:
#if NASL_DEBUG > 0
      fprintf(stderr, "nasl_read_var_ref: variable '%s' is undefined %s\n",
	      get_var_name(v), get_line_nb(tc));
#endif
      if (nasl_trace_fp != NULL)
	fprintf(nasl_trace_fp, "undef\n", v->var_type);
      break;

    default:
      fprintf(stderr, "nasl_read_var_ref: unhandled variable type %d\n",
	      v->var_type);
      if (nasl_trace_fp != NULL)
	fprintf(nasl_trace_fp, "???? (Var type %d)\n", v->var_type);
      break;
    }
  deref_cell(ret);
  return NULL;
}


tree_cell*
nasl_incr_variable(lex_ctxt* lexic, tree_cell* tc, int pre, int val)
{
  nasl_var	*v;
  int		old_val = 0, new_val;
  tree_cell	*retc;

  if (tc->type != REF_VAR)
    {
      fprintf(stderr,
	      "nasl_incr_variable: argument (type=%d) is not REF_VAR %s\n",
	      tc->type, get_line_nb(tc));
      return NULL;
    }

  v = tc->x.ref_val;

  switch (v->var_type)
    {
    case VAR2_INT:
      old_val = v->v.i_val;
      break;
    case VAR2_STRING:
    case VAR2_DATA:
#if NASL_DEBUG > 0
      fprintf(stderr, "nasl_incr_variable: variable %s is a STRING %s - converting to integer\n", get_var_name(v), get_line_nb(tc));
#endif
      old_val = v->v.str_val == NULL ? 0 : atoi(v->v.str_val);
      break;
    case VAR2_UNDEF:
#if NASL_DEBUG > 0
      fprintf(stderr, "nasl_incr_variable: variable %s is undefined %s\n",
	      get_var_name(v), get_line_nb(tc));
#endif
      old_val = 0;
      break;

    default:
      fprintf(stderr, "nasl_incr_variable: variable %s has bad type %d %s\n",
	      get_var_name(v), get_line_nb(tc));
      return NULL;
    }
  new_val = old_val + val;

  clear_var(v);
  v->var_type = VAR2_INT;
  v->v.i_val = new_val;

  retc = alloc_tree_cell(0, NULL);
  retc->type = CONST_INT;
  retc->x.i_val = pre ? new_val : old_val;
  
  return retc;  
}


static int	var2int(nasl_var* v, int defval)
{
  if (v == NULL)
    return defval;

  switch (v->var_type)
    {
    case VAR2_INT:
      return v->v.i_val;

    case VAR2_STRING:
    case VAR2_DATA:
      return atoi(v->v.str_val);

    case VAR2_UNDEF:
#if NASL_DEBUG > 0
      fprintf(stderr, "var2int: variable '%s' is undefined!\n", 
	      get_var_name(v));
#endif
    case VAR2_ARRAY:
    default:
      return defval;
    }
}

const char*	var2str(nasl_var* v)
{
  static char	s1[16];

  if (v == NULL)
    return NULL;

  switch (v->var_type)
    {
    case VAR2_INT:
      sprintf(s1, "%d", v->v.i_val);
      return s1;		/* dangerous if called twice in a row */

    case VAR2_STRING:
    case VAR2_DATA:
      return v->v.str_val;

    case VAR2_UNDEF:
#if NASL_DEBUG > 0
      fprintf(stderr, "var2str: variable '%s' is undefined!\n", 
	      get_var_name(v));
#endif
    case VAR2_ARRAY:
    default:
      return NULL;
    }
}

int
get_int_var_by_num(lex_ctxt* lexic, int num, int defval)
{
  nasl_var	*v = get_var_ref_by_num(lexic, num);
  return  var2int(v, defval);
}

int
get_int_var_by_name(lex_ctxt* lexic, const char* name, int defval)
{
  nasl_var	*v = get_var_ref_by_name(lexic, name, 1);
  return  var2int(v, defval);
}

int 
get_int_local_var_by_name(lex_ctxt * lexic, const char * name, int defval)
{
 nasl_var 	*v = get_var_ref_by_name(lexic, name, 0);
 return var2int(v, defval);
}


char*
get_str_var_by_num(lex_ctxt* lexic, int num)
{
  nasl_var	*v = get_var_ref_by_num(lexic, num);
  return  (char*)var2str(v);
}

char*
get_str_var_by_name(lex_ctxt* lexic, const char* name)
{
  nasl_var	*v = get_var_ref_by_name(lexic, name, 1);
  return  (char*)var2str(v);
}

char *
get_str_local_var_by_name(lex_ctxt * lexic, const char * name)
{
  nasl_var	*v = get_var_ref_by_name(lexic, name, 0);
  return  (char*)var2str(v);
}




int
get_var_size_by_name(lex_ctxt* lexic, const char* name)
{
  nasl_var	*v = get_var_ref_by_name(lexic, name, 1);
  return v == NULL ? 0 : v->var_size;
}

int
get_local_var_size_by_name(lex_ctxt * lexic, const char * name)
{
 nasl_var 	*v = get_var_ref_by_name(lexic, name, 0);
 return v == NULL ? 0 : v->var_size;
}

int
get_var_size_by_num(lex_ctxt* lexic, int num)
{
  nasl_var	*v = get_var_ref_by_num(lexic, num);
  return v == NULL ? 0 : v->var_size;
}


int
get_var_type_by_num(lex_ctxt* lexic, int num)
{
  nasl_var	*v = get_var_ref_by_num(lexic, num);
  return v == NULL ? 0 : v->var_type;
}

int
get_var_type_by_name(lex_ctxt* lexic, const char * name)
{
  nasl_var	*v = get_var_ref_by_name(lexic, name, 1);
  return v == NULL ? 0 : v->var_type;
}

int get_local_var_type_by_name(lex_ctxt * lexic, const char * name)
{
 nasl_var	*v = get_var_ref_by_name(lexic, name, 0);
 return v == NULL ? 0 : v->var_type;
}

nasl_iterator
nasl_array_iterator(tree_cell* c)
{
  nasl_var	*v;
  nasl_iterator	it;

  it.a = NULL;
  it.v = NULL;
  it.i = VAR_NAME_HASH;
  
  if (c == NULL || c == FAKE_CELL)
    return it;

  if (c->type == REF_VAR)
    {
      v = c->x.ref_val;
      if (v == NULL || v->var_type != VAR2_ARRAY)
	return it;
      it.a = v->v.a_val;
    }
  else if (c->type == REF_ARRAY || c->type == DYN_ARRAY)
    {
      it.a =c->x.ref_val;
    }
  else
    {
      fprintf(stderr, "nasl_array_iterator: unhandled type %d (0x%x)\n",
	      c->type, c->type);
      return it;
    }

      it.i = -1; 
      it.v = NULL;
      return it;

}

tree_cell*
nasl_iterate_array(nasl_iterator* it)
{
  nasl_var	*v;

  if (it == NULL)
    return NULL;

  if (it->v == NULL)
    while (++ it->i < VAR_NAME_HASH && (it->v = it->a[it->i]) == NULL)
      ;
  if (it->v == NULL)
    return NULL;

  v = it->v;
  it->v = v->next_var;
  return var2cell(v);
}

void
add_var_to_list(nasl_var** pv, int i, const nasl_var* v)
{
  int		h = hash_int(i);
  nasl_var	*v2;

  v2 = dup_var(v);
  if (v2 == NULL)
    return;
  v2->var_num = i;
  efree(&v2->var_name);
  v2->next_var = pv[h];
  pv[h] = v2;
  return;
}

